@@ -1,5 +1,5 @@
 
-__version__ = "3.6.1"
+__version__ = "3.7"
 
 # Hard coded commands
 waitCommandName = "wait for"
@@ -24,15 +24,20 @@ class UserEvent:
         self.name = name
     def shouldRecord(self, *args): # pragma: no cover - just documenting interface
         return True
-    def delayLevel(self):
+    def delayLevel(self, *args):
         return False
     def outputForScript(self, *args): # pragma: no cover - just documenting interface
         return self.name
     def generate(self, argumentString): # pragma: no cover - just documenting interface
-        raise UseCaseScriptError, "Don't know how to generate for " + repr(self.outputForScript(argumentString))
+        raise UseCaseScriptError("Don't know how to generate for " + repr(self.outputForScript(argumentString)))
     def isStateChange(self):
         # If this is true, recorder will wait before recording and only record if a different event comes in
         return False
     def implies(self, stateChangeLine, stateChangeEvent, *args):
         # If this is true, recorder will not record the state change if immediately followed by this event
         return self is stateChangeEvent
+    def checkPreviousWhenRejected(self):
+        # If this is true, 'implies' will be called even when 'shouldRecord' returns false
+        # Historical assumption has been that 'shouldRecord' often takes out events that are totally irrelevant
+        # But in some circumstances they can still be relevant in that sense
+        return False
