@@ -25,9 +25,9 @@ class Describer(storytext.guishared.Describer):
         self.logger.debug("Describing with updates...")
         stateChanges = self.findStateChanges()
         stateChangeWidgets = [ widget for widget, old, new in stateChanges ]
-        self.describeAppearedWidgets(stateChangeWidgets)
+        describedForAppearance = self.describeAppearedWidgets(stateChangeWidgets)
         stateChanges = self.describeStateChangeGroups(stateChangeWidgets, stateChanges)
-        self.describeStateChanges(stateChanges)
+        self.describeStateChanges(stateChanges, describedForAppearance)
         self.widgetsAppeared = []
         self.logger.debug("Finished describing with updates")
 
@@ -413,7 +413,8 @@ class Describer(storytext.guishared.Describer):
                isinstance(scrollPane.getViewport().getView(), swing.JTable)
 
     def getMaxDescriptionWidth(self, widget):
-        return None if self.isTableScrollPane(widget) or self.usesGrid(widget) else 130
+        return None if self.isTableScrollPane(widget) or self.usesGrid(widget) else \
+            storytext.guishared.Describer.getMaxDescriptionWidth(self, widget)
 
     def usesGrid(self, widget):
         return isinstance(widget.getLayout(), (awt.GridLayout, awt.GridBagLayout))
