@@ -1,12 +1,13 @@
 
-import storytext.guishared, util, types, os, logging
+import storytext.guishared, util, types, logging, sys
 from storytext.definitions import UseCaseScriptError
 from storytext.gridformatter import GridFormatter
-from storytext import applicationEvent
 from org.eclipse import swt
+from browserhtmlparser import BrowserHtmlParser
 from java.util import Date
-import xml.sax
-        
+from java.io import File, FilenameFilter
+from org.eclipse.jface.resource import ImageDescriptor
+
 class Describer(storytext.guishared.Describer):
     styleNames = [ (swt.widgets.CoolItem, []),
                    (swt.widgets.Item    , [ "SEPARATOR", "DROP_DOWN", "CHECK", "CASCADE", "RADIO" ]),
@@ -16,7 +17,7 @@ class Describer(storytext.guishared.Describer):
                    (swt.widgets.Text    , [ "PASSWORD", "SEARCH", "READ_ONLY" ]) ]
     ignoreWidgets = [ types.NoneType ]
     # DateTime children are an implementation detail
-    # Coolbars and Expandbars describe their children directly : they have two parallel children structures    
+    # Coolbars and Expandbars describe their children directly : they have two parallel children structures
     ignoreChildren = (swt.widgets.CoolBar, swt.widgets.ExpandBar, swt.widgets.DateTime)
     statelessWidgets = [ swt.widgets.Sash ]
     stateWidgets = [ swt.widgets.Shell, swt.widgets.Button, swt.widgets.Menu, swt.widgets.Link,
@@ -33,6 +34,9 @@ class Describer(storytext.guishared.Describer):
         self.widgetsDescribed = set()
         self.browserStates = {}
         self.clipboardText = None
+        self.imageDescriptorToNames = {}
+        self.imageToName = {}
+        self.handleImages()
         storytext.guishared.Describer.__init__(self)
 
     def setWidgetPainted(self, widget):
@@ -66,15 +70,15 @@ class Describer(storytext.guishared.Describer):
             if self.structureLogger.isEnabledFor(logging.DEBUG):
                 for widget in stateChangeWidgets:
                     self.describeStructure(widget)
-            self.describeAppearedWidgets(stateChangeWidgets)
-            self.describeStateChanges(stateChanges)
+            describedForAppearance = self.describeAppearedWidgets(stateChangeWidgets)
+            self.describeStateChanges(stateChanges, describedForAppearance)
         if shell is not None:
             self.describeClipboardChanges(shell.getDisplay())
             self.describe(shell)
         self.widgetsAppeared = []
         
     def shouldCheckForUpdates(self, widget, shell):
-        return not isinstance(widget, swt.widgets.Menu) or widget.getShell() == shell
+        return not isinstance(widget, swt.widgets.Menu) or (not widget.isDisposed() and widget.getShell() == shell)
 
     def widgetShowing(self, widget):
         return not widget.isDisposed() and util.isVisible(widget)
@@ -107,27 +111,28 @@ class Describer(storytext.guishared.Describer):
         return shell.getText()
 
     def getAllItemDescriptions(self, itemBar, indent=0, subItemMethod=None,
-                               prefix="", selection=[], columnCount=0):
+                               prefix="", selection=[], columnCount=0, **kw):
         descs = []
         for item in itemBar.getItems():
             currPrefix = prefix + " " * indent * 2
-            itemDesc = self.getItemDescription(item, currPrefix, item in selection)
+            selected = item in selection
             if columnCount:
-                row = [ itemDesc ]
-                for colIndex in range(1, columnCount):
-                    row.append(self.getItemColumnDescription(item, colIndex))
+                row = [ self.getItemColumnDescription(item, i, currPrefix, selected) for i in range(columnCount) ]
                 descs.append(row)
-            elif itemDesc:
-                descs.append(itemDesc)
+            else:
+                itemDesc = self.getItemDescription(item, currPrefix, selected)
+                if itemDesc:
+                    descs.append(itemDesc)
             if subItemMethod:
-                descs += subItemMethod(item, indent, prefix=prefix, selection=selection, columnCount=columnCount)
+                descs += subItemMethod(item, indent, prefix=prefix, selection=selection, columnCount=columnCount, **kw)
         return descs
 
-    def getCascadeMenuDescriptions(self, item, indent, **kw):
+    def getCascadeMenuDescriptions(self, item, indent, storeStatesForSubMenus=False, **kw):
         cascadeMenu = item.getMenu()
         if cascadeMenu:
-            descs = self.getAllItemDescriptions(cascadeMenu, indent+1, subItemMethod=self.getCascadeMenuDescriptions, **kw)
-            if indent == 1:
+            descs = self.getAllItemDescriptions(cascadeMenu, indent+1, subItemMethod=self.getCascadeMenuDescriptions, 
+                                                storeStatesForSubMenus=storeStatesForSubMenus, **kw)
+            if indent == 1 and storeStatesForSubMenus:
                 self.widgetsWithState[cascadeMenu] = "\n".join(descs)
             return descs
         else:
@@ -161,15 +166,15 @@ class Describer(storytext.guishared.Describer):
         else:
             return ""
 
-    def getMenuDescription(self, menu, indent=1):
-        return self.getItemBarDescription(menu, indent=indent, subItemMethod=self.getCascadeMenuDescriptions)
+    def getMenuDescription(self, menu, indent=1, **kw):
+        return self.getItemBarDescription(menu, indent=indent, subItemMethod=self.getCascadeMenuDescriptions, **kw)
     
     def getMenuState(self, menu):
         return self.getMenuDescription(menu, indent=2)
     
     def getMenuBarDescription(self, menubar):
         if menubar:
-            return "Menu Bar:\n" + self.getMenuDescription(menubar)
+            return "Menu Bar:\n" + self.getMenuDescription(menubar, storeStatesForSubMenus=True)
         else:
             return ""
 
@@ -201,7 +206,35 @@ class Describer(storytext.guishared.Describer):
     def getImageDescription(self, image):
         # Seems difficult to get any sensible image information out, there is
         # basically no query API for this in SWT
-        return "Image " + self.imageCounter.getId(image)
+        if self.imageDescriptionType == "name":
+            return self.getImageNameDescription(image)
+        else:
+            return self.getDefaultImageDescription(image)
+        
+    def getImageNameDescription(self, image):
+        desc = self.getImageName(image)
+        if desc is not None:
+            return "Icon '" + desc + "'"
+        else:
+            return "Unknown Image"
+
+    def getDefaultImageDescription(self, image):
+        name = "Image"
+        if self.imageDescriptionType == "number":
+            name += " " + self.imageCounter.getId(image)
+        return name
+    
+    def getImageName(self, image):
+        name = self.imageToName.get(image)
+        if name is not None:
+            return name
+        for img in sorted(self.imageDescriptorToNames, key=lambda x: self.imageDescriptorToNames[x]):
+            newImage = img.createImage()
+            if newImage.getImageData().data == image.getImageData().data:
+                name = self.imageDescriptorToNames.get(img)                
+                self.imageToName[image] = name                
+                break
+        return name
 
     def getCanvasDescription(self, widget):
         return "Canvas " + self.canvasCounter.getId(widget)
@@ -221,11 +254,19 @@ class Describer(storytext.guishared.Describer):
                 descs.append(tryStyle.lower().replace("_", " ").replace("separator", "---"))
         return descs
 
-    def getItemColumnDescription(self, item, colIndex):
+    def getItemColumnDescription(self, item, colIndex, prefix, *args):
         elements = [ item.getText(colIndex) ]
-        if item.getImage(colIndex):
-            elements.append(self.getImageDescription(item.getImage(colIndex)))
-        return self.combineElements(elements)
+        if colIndex:
+            if item.getImage(colIndex):
+                elements.append(self.getImageDescription(item.getImage(colIndex)))
+        else:
+            elements += self.getPropertyElements(item, *args)
+        
+        desc = self.combineElements(elements)
+        if desc and colIndex == 0:
+            return prefix + desc
+        else:
+            return desc
 
     def getControlDecoration(self, item):
         listener = self.getControlDecorationListener(item)
@@ -248,8 +289,10 @@ class Describer(storytext.guishared.Describer):
        
     def getControlDecorationDescription(self, item):
         deco = self.getControlDecoration(item)
-        if deco and self.decorationVisible(deco):
-            text = "Decoration " + self.getImageDescription(deco.getImage())
+        if deco:
+            imgDesc = self.getControlDecorationImageDescription(deco.getImage())
+        if deco and self.decorationVisible(deco): 
+            text = "Decoration " + imgDesc
             desc = deco.getDescriptionText()
             if desc:
                 text += "\n'" + desc + "'"
@@ -259,10 +302,19 @@ class Describer(storytext.guishared.Describer):
         if hasattr(deco, "isVisible"): # added in 3.6
             return deco.isVisible()
 
-        method = deco.getClass().getDeclaredMethod("shouldShowDecoration", None)
+        # Workaround for reflection bug in Jython 2.5.1
+        args = (None,) if sys.version_info[:3] <= (2, 5, 1) else ()
+        method = deco.getClass().getDeclaredMethod("shouldShowDecoration", *args)
         method.setAccessible(True)
-        return method.invoke(deco, None)
+        return method.invoke(deco, *args)
 
+    def getControlDecorationImageDescription(self, image):
+        # Hard coded. It gets indeterministic results
+        if self.imageDescriptionType == "name":
+            return "Unknown Image"
+        else:
+            return self.getDefaultImageDescription(image)
+        
     def getPropertyElements(self, item, selected=False):
         elements = []
         decoText = self.getControlDecorationDescription(item)
@@ -282,7 +334,10 @@ class Describer(storytext.guishared.Describer):
 
     def getLabelState(self, label):
         if label.getStyle() & swt.SWT.SEPARATOR:
-            return "-" * 10
+            if label.getStyle() & swt.SWT.VERTICAL:
+                return "-" * 5 + "vertical" + "-" * 5
+            else:
+                return "-" * 10
         elements = []
         if label.getText():
             elements.append("'" + label.getText() + "'")
@@ -333,7 +388,10 @@ class Describer(storytext.guishared.Describer):
 
     def getBrowserState(self, widget):
         url = util.getRealUrl(widget)
-        return url or BrowserHtmlParser().parse(widget.getText())
+        # Ignore non-breaking spaces, they are invisible anyway
+        rawText = widget.getText().replace(u"\xa0", " ").replace("&nbsp;", " ")
+        text = storytext.guishared.encodeToLocale(rawText)
+        return url or BrowserHtmlParser().parse(text)
         
     def getUpdatePrefix(self, widget, oldState, state):
         if isinstance(widget, (self.getTextEntryClass(), swt.browser.Browser)):
@@ -343,7 +401,9 @@ class Describer(storytext.guishared.Describer):
         elif util.getTopControl(widget):
             return "\n"
         elif isinstance(widget, swt.widgets.Menu):
-            return "\nUpdated " + widget.getParentItem().getText() + " Menu:\n"
+            parentItem = widget.getParentItem()
+            menuName = parentItem.getText() if parentItem else "Context"
+            return "\nUpdated " + menuName + " Menu:\n"
         elif isinstance(widget, (swt.widgets.Label, swt.custom.CLabel)) and len(state) == 0:
             return "\nLabel now empty, previously " + oldState
         else:
@@ -368,10 +428,6 @@ class Describer(storytext.guishared.Describer):
         desc = self.addHeaderAndFooter(widget, contents)
         return self.combineElements([ desc ] + properties)
 
-    def addHeaderAndFooter(self, widget, text):
-        header = "=" * 10 + " " + widget.__class__.__name__ + " " + "=" * 10
-        return header + "\n" + self.fixLineEndings(text.rstrip()) + "\n" + "=" * len(header)
-
     def getComboDescription(self, widget):
         return self.getTextDescription(widget)
 
@@ -481,7 +537,7 @@ class Describer(storytext.guishared.Describer):
         positions = []
         for child in children:
             if isinstance(child, swt.widgets.Sash) and child.getStyle() & swt.SWT.VERTICAL:
-                 positions.append(child.getLocation().x)
+                positions.append(child.getLocation().x)
         return sorted(positions)
 
     def layoutSortsChildren(self, widget):
@@ -525,13 +581,15 @@ class Describer(storytext.guishared.Describer):
     def formatContextMenuDescriptions(self):
         text = ""
         for contextMenu, menuId in self.contextMenuCounter.getWidgetsForDescribe():
-            text += "\n\nContext Menu " + str(menuId) + ":\n" + self.getMenuDescription(contextMenu)
+            menuDesc = self.getMenuDescription(contextMenu)
+            text += "\n\nContext Menu " + str(menuId) + ":\n" + menuDesc
+            self.widgetsWithState[contextMenu] = self.getMenuState(contextMenu) 
         return text
 
-    def getHorizontalSpan(self, widget, *args):
+    def getHorizontalSpan(self, widget, columns):
         layout = widget.getLayoutData()
         if hasattr(layout, "horizontalSpan"):
-            return layout.horizontalSpan
+            return min(layout.horizontalSpan, columns)
         else:
             return 1
 
@@ -550,35 +608,43 @@ class Describer(storytext.guishared.Describer):
     def getRawDataLayoutDetails(self, layout, *args):
         return [ str(layout.numColumns) + " columns" ] if hasattr(layout, "numColumns") else []
 
-class BrowserHtmlParser(xml.sax.ContentHandler):
-    def __init__(self):
-        xml.sax.ContentHandler.__init__(self)
-        self.text = ""
-        self.grid = []
-        self.activeElements = set()
-        
-    def parse(self, text):
-        xml.sax.parseString(text, self)
-        return self.text
+    def handleImages(self):
+        if self.imageDescriptionType:
+            self.buildImages()
+    
+    def buildImages(self):
+        self.buildImagesFromPaths()
 
-    def startElement(self, rawname, attrs):
-        name = rawname.lower()
-        self.activeElements.add(name)
-        if name == "tr":
-            self.grid.append([])
-        elif name == "td":
-            self.grid[-1].append("")
+    def buildImagesFromPaths(self):
+        for path in self.imagePaths:
+            self.findFiles(File(path))
 
-    def endElement(self, rawname):
-        name = rawname.lower()
-        self.activeElements.remove(name)
-        if name == "table":
-            formatter = GridFormatter(self.grid, max((len(r) for r in self.grid)), allowOverlap=False)
-            self.text += str(formatter)
-            self.grid = []
+    def getImageFiles(self, path):
+        d = File(path)
+        class Filter(FilenameFilter):
+            def accept(lself, d, fileName):#@NoSelf
+                return fileName.endswith(".gif") or fileName.endswith(".png") or fileName.endswith(".jpg")
 
-    def characters(self, content):
-        if "td" in self.activeElements:
-            self.grid[-1][-1] += content.rstrip()
-        elif "body" in self.activeElements:
-            self.text += content.rstrip()
+        return d.listFiles(Filter())
+    
+    def makeImageDescriptor(self, url):
+        imgDesc = ImageDescriptor.createFromURL(url)
+        name = url.getFile().split("/")[-1]
+        if imgDesc is not None:
+            #print "FILE", url.getFile()
+            self.imageDescriptorToNames[imgDesc] = name
+
+    def findFiles(self, pathAsFile):
+        if pathAsFile.isFile() and self.isImageType(pathAsFile.getName()):
+            self.makeImageDescriptor(pathAsFile.toURI().toURL())
+        elif pathAsFile.isDirectory():
+            for f in pathAsFile.listFiles():
+                if f is not None:
+                    self.findFiles(f)
+
+    def isImageType(self, fileName):
+        return fileName.endswith(".gif") or fileName.endswith(".png") or fileName.endswith(".jpg")
+
+    def checkWindow(self, window):
+        # Don't describe tooltips
+        return (window.getStyle() & swt.SWT.TOOL) == 0
