@@ -1,7 +1,8 @@
 
 """ Basic engine class, inherited in guishared.py and implemented by each GUI toolkit code """
 
-import recorder, replayer, os, sys, imp
+import recorder, replayer
+import os, sys, imp
 
 try:
     # In Py 2.x, the builtins were in __builtin__
@@ -22,9 +23,9 @@ class ScriptEngine:
         self.enableShortcuts = enableShortcuts
         self.recorder = recorder.UseCaseRecorder()
         self.replayer = self.createReplayer(**kwargs)
-        self.replayer.tryRunScript()
         self.registerShortcuts()
-
+        self.replayer.tryRunScript()
+        
     def recorderActive(self):
         return self.enableShortcuts or self.recorder.isActive()
 
@@ -51,8 +52,8 @@ class ScriptEngine:
                 shortcuts.append(replayer.ReplayScript(fullPath))
         return shortcuts
 
-    def createReplayer(self, **kwargs):
-        return replayer.UseCaseReplayer()
+    def createReplayer(self, **kw):
+        return replayer.UseCaseReplayer(**kw)
 
     def applicationEvent(self, name, category=None, supercedeCategories=[], timeDelay=0.001, delayLevel=0):
         # Small time delay to avoid race conditions: see replayer
@@ -76,9 +77,13 @@ class ScriptEngine:
         if len(args) == 0:
             return False
         else:
+            self.handleAdditionalOptions(options)
             self.runSystemUnderTest(args)
             return True
 
+    def handleAdditionalOptions(self, options):
+        pass
+    
     def runSystemUnderTest(self, args):
         # By default, just assume it's a python program. Allow this to be overridden
         self.run_python_file(args)
